/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_config.h,v 1.7.2.2 2000/03/13 09:46:25 john Exp $
 */

#ifndef __EDIR_CONFIG_H__
#define __EDIR_CONFIG_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#define EDIR_CONF_DEFAULT_CONFIG_FILE "/usr/local/etc/enhydra_director.conf"
#define EDIR_CONF_DEFAULT_DAEMON_FILE "/usr/local/sbin/edir_daemon"

/*
 * Define a set of known integer types.
 */
typedef unsigned char  octet;
typedef unsigned short uint16;
typedef short          int16;
typedef unsigned long  uint32;
typedef long           int32;

#ifndef WIN32 /* If using a real OS */

#include "config.h"

#include <stdio.h>

#ifdef HAVE_TYPES_H
#include <types.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_STRING_H
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
# ifndef HAVE_STRCHR 
#  define strchr index
#  define strrchr rindex
char *strchr(), *strrchr();
# endif
# ifndef HAVE_MEMCPY 
#  define memcpy(d, s, n) bcopy((s), (d), (n));
#  define memmove(d, s, n) bcopy((s), (d), (n));
# endif
#endif

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_STDARG_H
# include <stdarg.h>
#else
# ifdef HAVE_VARARGS_H
#  include <varargs.h>
#  define USE_VARARGS
# endif
#endif

#ifdef HAVE_SOCKET_H
# include <socket.h>
#endif

#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_IN_H
#include <arpa/inet.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_SYS_ERRNO_H
#include <sys/errno.h>
#endif

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#if defined(HAVE_TIME_H) && defined(HAVE_SYS_TIME_H)
# if defined(TIME_WITH_SYS_TIME)
#  include <sys/time.h>
#  include <time.h>
# else
#  ifdef TM_IN_SYS_TIME
#   include <sys/time.h>
#  else
#   include <time.h>
#  endif
# endif 
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# endif
# ifdef HAVE_TIME_H
#  include <time.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#if defined(HAVE_SIGNAL_H)
#include <signal.h>
#elif defined(HAVE_SYS_SIGNAL_H)
#include <sys/signal.h>
#endif

#ifdef HAVE_SYS_UIO_H
#include <sys/uio.h>
#endif

#ifdef XP_UNIX
#define EDIR_SOCKET		void *
#else
#define EDIR_SOCKET		int
#endif
#define EDIR_SOCKET_ERROR		-1
#define EDIR_SOCKET_INIT()	/* NO-OP */
#define EDIR_NULL_SOCKET		-1
#define EDIR_SOCKET_ISNULL(S)	((S) < 0)
#define EDIR_ADDR_ANY             NULL
#define EDIR_PORT_ANY		0

#ifdef HAVE_VSNPRINTF
# define edir_vsnprintf vsnprintf
#else
# error Need vsnprintf
#endif

#ifdef HAVE_SNPRINTF
# define edir_snprintf snprintf
#else
# error Need snprintf
#endif

#else /* Using windoze */

/**
 * Definitions for Windows Sockets.
 */
/* Not like they document this anywhere :( */
/* #define WIN32_LEAN_AND_MEAN */
#include <winsock2.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <time.h>

/**
 * Only multithreaded environments (ISAPI, NSAPI) supported
 * under Win32.
 */
#ifndef EDIR_MULTITHREADED
#define EDIR_MULTITHREADED 1
#endif

#define EDIR_SOCKET		SOCKET 
#define EDIR_SOCKET_ERROR	SOCKET_ERROR
#define EDIR_SOCKET_INIT()	edir_win_socket_init()
#define EDIR_NULL_SOCKET	-1
#define EDIR_SOCKET_ISNULL(S)	((S) < 0)
#define EDIR_ADDR_ANY           NULL
#define EDIR_PORT_ANY		0

#define edir_vsnprintf _vsnprintf
#define edir_snprintf _snprintf

/* Win32 does not have syslog() */
#define LOG_NOTICE		1
#define LOG_ERR			2
#define LOG_DEBUG		3

/* Win32 does not have iovec, but WSABUF is equivalent */
#define iovec _WSABUF
#define iov_base buf
#define iov_len len
#define iovec_t WSABUF
#define iov_len_t u_long
#define iov_base_t char FAR *

#endif /* WIN32 */

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_CONFIG_H__ */
