dnl
dnl  Enhydra Java Application Server 
dnl  The Initial Developer of the Original Code is Lutris Technologies Inc. 
dnl  Portions created by Lutris are Copyright (C) 1997-2000 Lutris
dnl  Technologies Inc. 
dnl  All Rights Reserved. 
dnl
dnl  The contents of this file are subject to the Enhydra Public License
dnl  Version 1.0 (the "License"); you may not use this file except in
dnl  compliance with the License. You may obtain a copy of the License at
dnl  http://www.enhydra.org/software/license/epl.html 
dnl
dnl  Software distributed under the License is distributed on an "AS IS"
dnl  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
dnl  License for the specific language governing rights and limitations
dnl  under the License. 
dnl
dnl  $Id: accommon.m4,v 1.2.2.1 2000/03/29 06:57:59 markd Exp $
dnl

dnl
dnl M4 macros and code shared by all configure.in files.  This defines
dnl a macro EDIR_COMMON while inserts the common code.
dnl

AC_DEFUN(EDIR_COMMON, [
    AC_CONFIG_AUX_DIR(../config/scripts/unix)
    AM_INIT_AUTOMAKE(EnhydraDirector,1.0)
    AM_CONFIG_HEADER(config.h:config.h.in)

    dnl XML lib info
    LIBXML_MAJOR_VERSION=1
    LIBXML_MINOR_VERSION=8
    LIBXML_MICRO_VERSION=6
    LIBXML_VERSION=$LIBXML_MAJOR_VERSION.$LIBXML_MINOR_VERSION.$LIBXML_MICRO_VERSION
    LIBXML_VERSION_INFO=`expr $LIBXML_MAJOR_VERSION + $LIBXML_MINOR_VERSION`:$LIBXML_MICRO_VERSION:$LIBXML_MINOR_VERSION
    AC_SUBST(LIBXML_MAJOR_VERSION)
    AC_SUBST(LIBXML_MINOR_VERSION)
    AC_SUBST(LIBXML_MICRO_VERSION)
    AC_SUBST(LIBXML_VERSION)
    AC_SUBST(LIBXML_VERSION_INFO)

    dnl Checks for programs.

    AC_PATH_PROG(TEST,test,$PATH)

    AC_ARG_ENABLE(debugging,
    [  --enable-debugging      Enables debugging macros],
    [ DEFINE_DEBUG="-DDEBUG" ])
    AC_SUBST(DEFINE_DEBUG)

    AC_PROG_CC
    AC_PROG_INSTALL
    AC_PROG_CPP
    AC_AIX
    AC_PATH_PROG(RM, rm, /bin/rm)
    AC_PATH_PROG(MV, mv, /bin/mv)
    AC_PATH_PROG(TAR, tar, /bin/tar)

    dnl Checks for libraries.

    AC_CHECK_FUNC(connect)
    if test $ac_cv_func_connect = no; then
        AC_CHECK_LIB(socket, connect)
    fi

    AC_CHECK_FUNC(gethostbyname)
    if test $ac_cv_func_gethostbyname = no; then
        AC_CHECK_LIB(xnet, gethostbyname)
    fi

    AC_CHECK_FUNC(gethostent)
    if test $ac_cv_func_gethostent = no; then
        AC_CHECK_LIB(nsl, gethostent)
    fi

    AC_CHECK_FUNC(shmat)
    if test $ac_cv_func_shmat = no; then
        AC_CHECK_LIB(ipc, shmat)
    fi

    dnl Checks for header files.

    AC_HEADER_DIRENT
    AC_HEADER_STDC
    AC_TIME_WITH_SYS_TIME
    AC_CHECK_HEADERS(types.h sys/types.h string.h strings.h stdlib.h)
    AC_CHECK_HEADERS(stdarg.h varargs.h)
    AC_CHECK_HEADERS(socket.h sys/socket.h netinet/in.h arpa/inet.h)
    AC_CHECK_HEADERS(errno.h sys/errno.h sys/param.h netdb.h time.h sys/time.h)
    AC_CHECK_HEADERS(unistd.h signal.h sys/signal.h syslog.h sys/mman.h fcntl.h)
    AC_CHECK_HEADERS(sys/stat.h sys/file.h sys/uio.h wait.h sys/wait.h ctype.h)
    AC_CHECK_HEADERS(dirent.h malloc.h ieeefp.h nan.h fp_class.h float.h)
    AC_CHECK_HEADERS(sys/select.h)

    dnl Checks for typedefs, structures, and compiler characteristics.
    AC_C_CONST

    AC_EGREP_HEADER(semun, sys/sem.h, , ac_cv_need_union_semun=yes)

    if test -n "$ac_cv_need_union_semun" ; then
            AC_DEFINE(EDIR_NEED_UNION_SEMUN, 1)
    fi

    dnl Checks for library functions.
    AC_FUNC_STRFTIME
    AC_CHECK_FUNCS(strerror strtol strchr memcpy bcopy mmap snprintf vsnprintf)
    AC_CHECK_FUNCS(getcwd openlog syslog getdtablesize waitpid wait4 wait3 wait)
    AC_CHECK_FUNCS(setenv putenv strcasecmp strcmpi stricmp strncasecmp)
    AC_CHECK_FUNCS(strncmpi strincmp strndup strftime)
    AC_CHECK_FUNCS(finite isnand fp_class class fpclass localtime)
    AC_FUNC_SETPGRP

    dnl Checks for isnan in libm if not in libc
    AC_CHECK_FUNC(isnan, , AC_CHECK_LIB(m, isnan,
      [M_LIBS="-lm"; AC_DEFINE(HAVE_ISNAN)]))
    AC_SUBST(HAVE_ISNAN)

    AC_CHECK_FUNC(isinf, , AC_CHECK_LIB(m, isinf,
      [M_LIBS="-lm"; AC_DEFINE(HAVE_ISINF)]))
    AC_SUBST(HAVE_ISINF)

    CFLAGS="${CFLAGS} -DXML_USE_BUFFER_CONTENT"
    AC_SUBST(CFLAGS)

    AC_SUBST(M_LIBS)
])
