package discRack.business.disc;

import discRack.business.DiscRackBusinessException;
import discRack.business.person.Person;
import discRack.data.person.PersonDO;
import discRack.data.disc.DiscDO;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;
     
public class Disc
{
    protected DiscDO myDO = null;
         
    public Disc()
        throws DiscRackBusinessException
    {
        try {
            this.myDO = DiscDO.createVirgin();  
        } catch(DatabaseManagerException ex) {
            throw new DiscRackBusinessException("Error creating empty Disc", ex);
        } catch(ObjectIdException ex) {
            throw new DiscRackBusinessException("Error creating empty Disc", ex);
        }
    }
    
    protected Disc(DiscDO theDisc) 
        throws DiscRackBusinessException
    {
        this.myDO = theDisc;
    }
    
    public String getHandle()
        throws DiscRackBusinessException
    {
        try {
            return this.myDO.getHandle();
        } catch(DatabaseManagerException ex) {
            throw new DiscRackBusinessException("Error getting disc's handle", ex);
        }
    }
    
    public String getTitle() 
        throws DiscRackBusinessException
    {
        try {
            return myDO.getTitle();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's title", ex);
        }
    }
    
    public String getArtist()
        throws DiscRackBusinessException
    {
        try {
            return myDO.getArtist();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's artist", ex);
        }
    }
    
    public String getGenre()
        throws DiscRackBusinessException
    {
        try {
            return myDO.getGenre();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's genre", ex);
        }
    }
    
    public boolean isLiked()
        throws DiscRackBusinessException
    {
        try {
            return myDO.getIsLiked();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's likedness", ex);
        }
    }
    
    public void setTitle(String title) 
        throws DiscRackBusinessException
    {
        try {
            this.myDO .setTitle(title);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's title", ex);
        }
    }
    
	public void setArtist(String artist) 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.setArtist(artist);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's artist", ex);
        }
    }
    
	public void setGenre(String genre) 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.setGenre(genre);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's genre", ex);
        }
    }
    
    public void setOwner(Person owner) 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.setOwner(PersonDO.createExisting(owner.getHandle()));
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's owner", ex);
        }
    }
    
    public void setLiked(boolean isLiked) 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.setIsLiked(isLiked);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's likedness", ex);
        }
    }
    
    public void save() 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.commit();
        } catch(Exception ex) {
            throw new DiscRackBusinessException("Error saving disc", ex);
        }
    }
    
    public void delete() 
        throws DiscRackBusinessException
    {
        try {
            this.myDO.delete();
        } catch(Exception ex) {
            throw new DiscRackBusinessException("Error deleting disc", ex);
        }
    }
}
