/**--------------------------------------------------------------
* discRack demo
*
* @class: DiscFactoryImpl
* @version 
* @author  
* @since   
*
*
*
*
*--------------------------------------------------------------*/

package discRack.business.disc;

import discRack.business.DiscRackBusinessException;
import discRack.data.disc.*;
import discRack.business.person.Person;
import com.lutris.dods.builder.generator.query.*;
import discRack.business.person.Person;
import discRack.data.person.PersonDO;
import com.lutris.appserver.server.sql.ObjectId;

public class DiscFactory
{    
    /** 
     * The findDiscsForPerson method performs a database query to
     * return an array of <CODE>Disc</CODE> objects
     * representing all the rows in the <CODE>disc</CODE> table
     * that have and owner matching <CODE>Person owner</CODE>. 
     *
     * @exception DataObjectException
     *   If an object is not found in the database.
     */
    public static Disc[] findDiscsForPerson(Person owner) 
        throws DiscRackBusinessException 
    {
	    Disc[] theDiscArray = null;
        
	    try {
            DiscQuery query = new DiscQuery();
	        // To restrict the set of discs returned
	        // to the that of the specific owner set in the query
	        query.setQueryOwner(PersonDO.createExisting(owner.getHandle()));
            // Order discs alphabetically by artist
            query.addOrderByArtist();
	        DiscDO[] DOarray = query.getDOArray();
	        theDiscArray = new Disc[ DOarray.length ];
	        for ( int i = 0; i < DOarray.length; i++ )
	    	theDiscArray[i] = new Disc(DOarray[i]);
	    }catch(Exception ex) {
            throw new DiscRackBusinessException("Exception in findDiscsForPerson()", ex);
        }
        
        return theDiscArray;
    }
    
    /**
     * 
     */
    public static Disc findDiscByID(String id) 
        throws DiscRackBusinessException
    {
        Disc theDisc = null;
        
	    try {
            DiscQuery query = new DiscQuery();
	        // Require that we only find ONE disc
            query.requireUniqueInstance();
	        query.setQueryOId(new ObjectId(id));
	        DiscDO theDiscDO = query.getNextDO();
	    	theDisc = new Disc(theDiscDO);
            return theDisc;
	    }catch(Exception ex) {
            throw new DiscRackBusinessException("Exception in findDiscsForPerson()", ex);
        }
    }
}

