package discRack.presentation.personMgmt;

import discRack.business.person.*;
import discRack.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.*;
import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import discRack.presentation.DiscRackPresentationException;
import discRack.business.DiscRackBusinessException;

/**
 * Register.java handles the user registration functionality 
 *  of the DiscRack app.
 *
 */
public class Register extends BasePO 
{    
    /**
     * Constants
     */
    private static String LOGINNAME = "login";
    private static String PASSWORD = "password";
    private static String REPASSWORD = "repassword";
    private static String FIRSTNAME = "firstname";
    private static String LASTNAME = "lastname";

    /**
     * Superclass method override
     */
    public boolean loggedInUserRequired()
    {
        return false;
    }
    
    /**
     *  Default event. Just show the page.
     */
    public String handleDefault() 
        throws HttpPresentationException 
    {
	    return showPage(null);
    }

    /**
     *  process login data, save fields to PersonManager if correct, 
     *  otherwise output results
     *  save error results, 
     *  control PO flow
     */
    public String handleRegister() 
	     throws HttpPresentationException 
    {
	    String login = this.getComms().request.getParameter(LOGINNAME);
	    String password = this.getComms().request.getParameter(PASSWORD);
	    String firstname = this.getComms().request.getParameter(FIRSTNAME);;
	    String lastname = this.getComms().request.getParameter(LASTNAME);
	    String repassword = this.getComms().request.getParameter(REPASSWORD);
    
	    // if login or password field is empty, generate error and redirect to this PO
	    if (login.length() == 0 || password.length() ==0 || 
	        firstname.length() == 0 || lastname.length() == 0) {
            return showPage("Missing information. Please make sure all fields are filled out exactly");    
        }        
        
        //Check that password was duplicated correctly
	    if(!repassword.equals(password)) {
            return showPage("Please make sure your password and password confirmation match exactly");    
	    }
	
        try {
            // Now check that the login name is not taken.
	        if(null == PersonFactory.findPerson(login)) {
	            Person user = new Person();
	            user.setLogin(login);
	            user.setPassword(password);
	            user.setFirstname(firstname);
	            user.setLastname(lastname);
	            //Add the user to the database.
	            user.save();
                this.getSessionData().setUserMessage(
                     "You are registered, " + user.getFirstname() + ", please log in");
	            throw new ClientPageRedirectException(LOGIN_PAGE);
	        } else {
                // Login name already taken
                // Redirect to this same PO with informative error message
                return showPage("The login name " + login + " is already taken. Please try another.");    
	        }
        } catch(DiscRackBusinessException ex) {
            throw new DiscRackPresentationException("Exception logging in user: ", ex);
        }
    }
    
    /** 
     *  produce HTML for this PO
     */
    public String showPage(String errorMsg)
        throws HttpPresentationException 
    {
	    // instantiate XMLC object
	    RegisterHTML page = new RegisterHTML();
	
        if(null == errorMsg) {       
	        page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        } else {
            page.setTextErrorText(errorMsg);
        }
	    
        if(null != this.getComms().request.getParameter(LOGINNAME)) {
            page.getElementLogin().setValue(this.getComms().request.getParameter(LOGINNAME));
        }
        if(null != this.getComms().request.getParameter(FIRSTNAME)) {
            page.getElementFirstname().setValue(this.getComms().request.getParameter(FIRSTNAME));
        }
        if(null != this.getComms().request.getParameter(LASTNAME)) {
            page.getElementLastname().setValue(this.getComms().request.getParameter(LASTNAME));
        }
        
        return page.toDocument();
    }
}
